@@ -19,9 +19,9 @@ import os
 import traceback
 import logging
 from itertools import chain
-from PySide2.QtCore import *
-from PySide2.QtGui import *
-from PySide2.QtWidgets import *
+from PySide6.QtCore import *
+from PySide6.QtGui import *
+from PySide6.QtWidgets import *
 
 from . import ogAST, ogParser, sdlSymbols, genericSymbols, Renderer, Pr
 
@@ -61,12 +61,14 @@ def copy(selection):
     for item in branch_top_level + floating_items:
         branch_ast, terminators = copy_branch(item)
         COPY_PASTE.append((branch_ast, terminators))
-    LOG.debug('COPY-PASTE LIST:' + str(COPY_PASTE))
+    LOG.debug('COPY-PASTE LIST: %s', str(COPY_PASTE))
 
 
 def copy_branch(top_level_item):
     ''' Copy branches (recursively) '''
     res_terminators = []
+    # Parse without symbol ids, so that pos_x is kept with the right value
+    Pr.g_useSymbolId = False
     pr_text = '\n'.join(Pr.generate(top_level_item, cpy=True,
                                     nextstate=False, recursive=True))
     item_ast, terminators = top_level_item.get_ast(pr_text)
@@ -80,7 +82,7 @@ def copy_branch(top_level_item):
     # If top_level_item is an horizontal symbol, all its children (branch)
     # are automatically copied. In the case that several following symbols
     # are selected however, the followers of the top_level must be
-    # explicitely added to the copy list:
+    # explicitly added to the copy list:
     if not isinstance(top_level_item, genericSymbols.HorizontalSymbol):
         next_aligned = top_level_item.next_aligned_symbol()
         while next_aligned and next_aligned.grabber.isSelected():
@@ -132,6 +134,7 @@ def paste(parent, scene):
             common_name = shared[2]
             pr_text     = shared[3]
             #  Copy to the local clipboard
+            # LOG.debug("PASTE: " + pr_text)
             ast, _, _, _, terminators = \
                 ogParser.parseSingleElement(common_name, pr_text)
             COPY_PASTE.append(([ast], terminators))
@@ -231,12 +234,30 @@ def paste_below_item(parent, scene):
             # Create the new item from the AST description
             new_item = Renderer.render(i, scene=CLIPBOARD,
                                        parent=None, states=states)
+            if isinstance(new_item, sdlSymbols.Label) \
+                    and parent.next_aligned_symbol() is not None:
+                # When a transition starting from a Label is pasted, we must
+                # check that the last symbol is not a terminator unless the
+                # parent had no following symbol. This would clash otherwise.
+                follower = new_item.next_aligned_symbol()
+                while follower is not None:
+                    if isinstance(follower, (sdlSymbols.Join, sdlSymbols.State)):
+                        raise TypeError("Cannot paste here because of the terminator symbol")
+                    follower = follower.next_aligned_symbol()
+
             # Check that item is compatible with parent
             if (type(new_item).__name__ in parent.allowed_followers):
                 # Move the item from the clipboard to the scene
                 Renderer.add_to_scene(new_item, scene)
                 new_item.pos_x = new_item.pos_y = 0.0
                 symbols.append(new_item)
+                if isinstance(new_item, sdlSymbols.Label):
+                    # Labels are rendered as a transition, but we must add
+                    # all the individual signals to the list
+                    follower = new_item.next_aligned_symbol()
+                    while follower is not None:
+                        symbols.append(follower)
+                        follower = follower.next_aligned_symbol()
             else:
                 raise TypeError('Cannot paste here ({t1} cannot follow {t2})'
                                 .format(t1=type(new_item), t2=type(parent)))
