@@ -6,7 +6,7 @@
 
     SDL is the Specification and Description Language (Z100 standard from ITU)
 
-    Copyright (c) 2012-2020 European Space Agency
+    Copyright (c) 2012-2021 European Space Agency
 
     Designed and implemented by Maxime Perrotin
 
@@ -18,7 +18,7 @@
     be done on any scene (e.g. clipboard).
 
     There is a single rendering function for all SDL construct, and a dispatch
-    machanism (using the Python3-backported feature called singledispatch).
+    mechanism (using the Python3-backported feature called singledispatch).
 
     Rendering can be done for single elements (returns the symbol) or for
     complete diagrams.
@@ -71,7 +71,7 @@ def _block(ast, scene):
         if each.instance_of_ref:
             top_level.append(render(each.instance_of_ref, scene))
     for each in ast.parent.text_areas:
-        # Sytem level may contain text areas with signal definitions, etc.
+        # System level may contain text areas with signal definitions, etc.
         top_level.append(render(each, scene))
     if not ast.parent.text_areas:
         # If signals are declared outside from a textbox, create one
@@ -79,10 +79,12 @@ def _block(ast, scene):
            param=('(' + sig['type'].ReferencedTypeName.replace('-', '_') + ')')
                  if 'type' in sig else '')
              for sig in ast.parent.signals]
-        procedures = ["procedure {proc.inputString};\n{optfpar}external;\n"
+        procedures = ["{exported}procedure {proc.inputString};\n{optfpar}{external};\n"
                       .format(proc=proc,
-                              optfpar="fpar\n    " + u",\n    ".join
-                              ([u"{direc} {fp[name]} {asn1}"
+                              exported="exported " if proc.exported else "",
+                              external="referenced" if proc.referenced else "external",
+                              optfpar="fpar\n    " + ",\n    ".join
+                              (["{direc} {fp[name]} {asn1}"
                                 .format(fp=fpar,
                                         direc="in"
                                            if fpar['direction']=='in'
@@ -125,8 +127,8 @@ def _automaton(ast, scene):
     # Render procedures symbols
     top_level_symbols.extend(
             [render(proc, scene)
-                            for proc in ast.inner_procedures
-                            if not proc.external])
+                for proc in ast.inner_procedures
+               if not proc.external and not proc.textual_procedure])
 
     # Render the start symbol
     if ast.start:
@@ -276,7 +278,7 @@ def _transition(ast, scene, parent, states):
 
 @render.register(ogAST.Comment)
 def _comment(ast, scene, parent, states=None):
-    ''' Create a COMMENT symbol - note: relative positionning is lost '''
+    ''' Create a COMMENT symbol - note: relative positioning is lost '''
     _, _ = scene, states
     return genericSymbols.Comment(parent, ast=ast)
 
@@ -387,6 +389,8 @@ def _input(ast, scene, parent, states):
 @render.register(ogAST.ContinuousSignal)
 def _continuous_signal(ast, scene, parent, states):
     ''' Add continuous signal to the scene '''
+    if ast.artificial:
+        return None
     cont = sdlSymbols.ContinuousSignal(parent, ast=ast)
     if cont not in scene.items():
         add_to_scene(cont, scene)
