@@ -4,10 +4,10 @@
 """
     Graphical text editing functionalities:
         - Autocompletion
-        - Syntax highlighing
+        - Syntax highlighting
         - Automatic placement
 
-    Copyright (c) 2012-2019 European Space Agency
+    Copyright (c) 2012-2020 European Space Agency
 
     Designed and implemented by Maxime Perrotin for the TASTE project
 
@@ -17,9 +17,9 @@
 import string
 import logging
 
-from PySide2.QtCore import *
-from PySide2.QtGui import *
-from PySide2.QtWidgets import *
+from PySide6.QtCore import *
+from PySide6.QtGui import *
+from PySide6.QtWidgets import *
 from . import undoCommands
 
 __all__ = ['EditableText']
@@ -32,7 +32,7 @@ class Completer(QGraphicsProxyWidget):
     def __init__(self, parent):
         ''' Create an autocompletion list popup '''
         widget = QListWidget()
-        super(Completer, self).__init__(parent)
+        super().__init__(parent)
         self.setWidget(widget)
         self.string_list = QStringListModel()
         self._completer = QCompleter()
@@ -73,7 +73,7 @@ class Completer(QGraphicsProxyWidget):
 
     # pylint: disable=C0103
     def keyPressEvent(self, e):
-        super(Completer, self).keyPressEvent(e)
+        super().keyPressEvent(e)
         if e.key() == Qt.Key_Escape:
             self.parentItem().setFocus()
         # Consume the event so that it is not repeated at EditableText level
@@ -82,7 +82,7 @@ class Completer(QGraphicsProxyWidget):
     # pylint: disable=C0103
     def focusOutEvent(self, event):
         ''' When the user leaves the popup, return focus to parent '''
-        super(Completer, self).focusOutEvent(event)
+        super().focusOutEvent(event)
         self.hide()
         self.resize(0, 0)
         self.parentItem().setFocus()
@@ -93,40 +93,54 @@ class Highlighter(QSyntaxHighlighter):
     ''' Class for handling syntax highlighting in editable text '''
     def __init__(self, parent, blackbold_patterns, redbold_patterns):
         ''' Define highlighting rules - inputs = lists of patterns '''
-        super(Highlighter, self).__init__(parent)
+        super().__init__(parent)
         self.highlighting_rules = []
 
         # Black bold items (allowed keywords)
         black_bold_format = QTextCharFormat()
         black_bold_format.setFontWeight(QFont.Bold)
-        self.highlighting_rules = [(QRegExp(pattern, cs=Qt.CaseInsensitive),
-            black_bold_format) for pattern in blackbold_patterns]
+        for pattern in blackbold_patterns:
+            self.highlighting_rules.append(
+                    (QRegularExpression(pattern,
+                                        QRegularExpression.CaseInsensitiveOption),
+                                        black_bold_format))
 
         # Red bold items (reserved keywords)
         red_bold_format = QTextCharFormat()
         red_bold_format.setFontWeight(QFont.Bold)
         red_bold_format.setForeground(Qt.red)
+
         for pattern in redbold_patterns:
             self.highlighting_rules.append(
-                    (QRegExp(pattern, cs=Qt.CaseInsensitive), red_bold_format))
+                    (QRegularExpression(pattern,
+                                        QRegularExpression.CaseInsensitiveOption),
+                                        red_bold_format))
 
         # Comments
         comment_format = QTextCharFormat()
         comment_format.setForeground(Qt.darkBlue)
         comment_format.setFontItalic(True)
-        self.highlighting_rules.append((QRegExp('--[^\n]*'), comment_format))
+        self.highlighting_rules.append(
+                (QRegularExpression('--[^\n]*'), comment_format))
 
     # pylint: disable=C0103
     def highlightBlock(self, text):
         ''' Redefined function to apply the highlighting rules '''
         for expression, formatter in self.highlighting_rules:
-            index = expression.indexIn(text)
-            while (index >= 0):
-                length = expression.matchedLength()
-                self.setFormat(index, length, formatter)
-                index = expression.indexIn(text, index + length)
+            index = expression.globalMatch(text)
+            while (index.hasNext()):
+                match = index.next()
+                length = match.capturedLength()
+                self.setFormat(match.capturedStart(), length, formatter)
 
 
+def zValueItems(item):
+    ''' Find all items that require a zValue increase to ensure that both
+    the text and completer are visible. '''
+    if item.parent is not None:
+        yield from zValueItems(item.parent)
+    yield item
+
 # pylint: disable=R0902
 class EditableText(QGraphicsTextItem):
     '''
@@ -138,7 +152,7 @@ class EditableText(QGraphicsTextItem):
     word_under_cursor = Signal(str)
 
     def __init__(self, parent, text='...', hyperlink=None):
-        super(EditableText, self).__init__(parent)
+        super().__init__(parent)
         self.parent = parent
         self.setFont(QFont('Ubuntu', 10))
         self.completer = Completer(self)
@@ -212,18 +226,17 @@ class EditableText(QGraphicsTextItem):
 
     def try_resize(self):
         '''
-            If needed, request a resizing of the parent item
-            (when text size expands)
+            Request a resizing of the parent item (symbol) when text changes
         '''
-        if self.parent.auto_expand:
-            self.setTextWidth(-1)
-            parent_rect = self.parent.boundingRect()
-            rect = self.boundingRect()
-            if rect.width() + 15 > parent_rect.width():
-                parent_rect.setWidth(rect.width() + 15)
-            parent_rect.setHeight(max(rect.height(), parent_rect.height()))
-            self.parent.resize_item(parent_rect)
-            self.set_textbox_position()
+        minW, minH = self.parent.min_size
+        self.setTextWidth(-1)
+        parent_rect = self.parent.boundingRect()
+        rect = self.boundingRect()
+        #if rect.width() + 15 > parent_rect.width():
+        parent_rect.setWidth(max(rect.width() + 15, minW))
+        parent_rect.setHeight(max(rect.height(), minH))
+        self.parent.resize_item(parent_rect)
+        self.set_textbox_position()
 
     @Slot(QListWidgetItem)
     def completion_selected(self, item):
@@ -252,7 +265,7 @@ class EditableText(QGraphicsTextItem):
 
     def context_completion_list(self, force=False):
         ''' Advanced context-dependent autocompletion '''
-        # Select text from the begining of a line to the cursor position
+        # Select text from the beginning of a line to the cursor position
         # Then keep the last word including separators ('!' and '.')
         # This word (e.g. variable!field!subfield) is then used to update
         # the autocompletion list.
@@ -278,7 +291,7 @@ class EditableText(QGraphicsTextItem):
         '''
             Activate the autocompletion window if relevant
         '''
-        super(EditableText, self).keyPressEvent(event)
+        super().keyPressEvent(event)
         # Typing Esc allows to stop editing text:
         if event.key() == Qt.Key_Escape:
             self.clearFocus()
@@ -321,6 +334,7 @@ class EditableText(QGraphicsTextItem):
 
             self.completer.setPos(pos_x, pos_y)
             self.completer.show()
+            self.completer.ensureVisible()
             self.force_focus = True   # avoid unwanted syntax checks
             # Make sure parent item has higher visibility than its siblings
             # (useful in decision branches)
@@ -342,7 +356,7 @@ class EditableText(QGraphicsTextItem):
         if self.completer.isVisible():
             self.completer.hide()
             self.completer.resize(0, 0)
-        super(EditableText, self).mousePressEvent(event)
+        super().mousePressEvent(event)
 
     # pylint: disable=C0103
     def focusOutEvent(self, event):
@@ -352,7 +366,7 @@ class EditableText(QGraphicsTextItem):
             that got the focus.
         '''
         if not self.editing:
-            return super(EditableText, self).focusOutEvent(event)
+            return super().focusOutEvent(event)
         if self.completer and not self.completer_has_focus:
             self.completer.hide()
             self.completer.resize(0, 0)
@@ -394,7 +408,7 @@ class EditableText(QGraphicsTextItem):
                     self.scene().undo_stack.push(undo_cmd)
                     try:
                         self.parent.cam(self.parent.pos(),
-                                              self.parent.pos())
+                                        self.parent.pos())
                     except AttributeError:
                         # Some parents may not have CAM function (e.g. Channel)
                         pass
@@ -404,20 +418,18 @@ class EditableText(QGraphicsTextItem):
                     self.scene().undo_stack.push(undo_cmd)
         self.set_text_alignment()
         # Reset Z-Values that were increased when getting focus
-        top_level = self.parent.top_level()
-        top_level.setZValue(top_level.zValue() - 1)
-        self.parent.setZValue(self.parent.zValue() - 1)
-        super(EditableText, self).focusOutEvent(event)
+        for each in zValueItems(self.parent):
+            each.setZValue(each.zValue() - 1)
+        super().focusOutEvent(event)
 
     # pylint: disable=C0103
     def focusInEvent(self, event):
         ''' When user starts editing text, save previous state for Undo '''
-        super(EditableText, self).focusInEvent(event)
+        super().focusInEvent(event)
         # Change the Z-value of items to make sure the
         # completer is always be on top of other symbols
-        top_level = self.parent.top_level()
-        top_level.setZValue(top_level.zValue() + 1)
-        self.parent.setZValue(self.parent.zValue() + 1)
+        for each in zValueItems(self.parent):
+            each.setZValue(each.zValue() + 1)
 
         # Trigger a select - side effect makes the toolbar update
         try:
