@@ -6,14 +6,14 @@
     that can be used to create diagrams.
 
     The Symbol class contains common behaviour shared by inherited
-    symobols - in particular the VerticalSymbol is a class of symbols
+    symbols - in particular the VerticalSymbol is a class of symbols
     that can only be vertically aligned ; Horizontal symbols, on the
     other hand, can be move in both directions.
 
     The "Comment" class of symbols is a floating symbol that is
     connected to a parent symbol on the right.
 
-    This library also contains the defintition of connections
+    This library also contains the definition of connections
     (the Connection class).
 
     Symbols can have an editable text area, which behaviour is
@@ -23,7 +23,7 @@
 
 
     Major functionalities offered by the generic Symbol classes are
-    the insersion and deletion of items (possibly recursively if there
+    the insertion and deletion of items (possibly recursively if there
     are child symbols), the moving and resizing, the collision
     avoidance manoeuvres (when moving a group of symbols on top
     of another group, it has the effect of "pushing" the colliders
@@ -41,7 +41,7 @@
     For a complete example, look at the "sdlSymbols.py" module, that
     provide symbol definitions that correspond to an SDL editor.
 
-    Copyright (c) 2012-2019 European Space Agency
+    Copyright (c) 2012-2020 European Space Agency
 
     Designed and implemented by Maxime Perrotin for the TASTE project
 
@@ -53,10 +53,10 @@ import os
 import logging
 import traceback
 
-from PySide2.QtCore import *
-from PySide2.QtGui import *
-from PySide2.QtWidgets import *
-from PySide2.QtUiTools import QUiLoader
+from PySide6.QtCore import *
+from PySide6.QtGui import *
+from PySide6.QtWidgets import *
+from PySide6.QtUiTools import QUiLoader
 
 from . import undoCommands, ogAST, ogParser
 from .Connectors import Connection, VerticalConnection, CommentConnection, \
@@ -88,8 +88,6 @@ class Symbol(QObject, QGraphicsPathItem):
     _conn_targets = []   # target types that can connect to this symbol
     # By default a symbol is resizeable
     resizeable = True
-    # By default symbol size may expand when inner text exceeds border
-    auto_expand = True
     # default size: "any" or "symbol_default" - optionally used at creation
     default_size = "symbol_default"
     # By default connections between symbols are lines, not arrows
@@ -128,12 +126,15 @@ class Symbol(QObject, QGraphicsPathItem):
     # the size in an unwanted manner.
     width, height = 0, 0
 
+    # Minimum size for symbol
+    min_size = (10, 10)
+
     def __init__(self, parent=None):
         '''
             Create top level symbol and propagate important properties
             from parent items
         '''
-        super(Symbol, self).__init__(parent)
+        super().__init__(parent)
         QGraphicsPathItem.__init__(self, parent)
         # Current mode, can be empty string, "Resize", or "Move"
         self.mode = ''
@@ -160,7 +161,7 @@ class Symbol(QObject, QGraphicsPathItem):
         # self.setCacheMode(QGraphicsItem.DeviceCoordinateCache)
         # Apply symbol default mouse cursor
         self.setCursor(self.default_cursor)
-        # De-ativate cache mode otherwise paint is not properly updated
+        # De-activate cache mode otherwise paint is not properly updated
         # for the comment symbol (need to refresh on_the_right property)
         self.setCacheMode(QGraphicsItem.NoCache)
         # Initialize variables used when moving/resizing
@@ -333,8 +334,8 @@ class Symbol(QObject, QGraphicsPathItem):
         # if check_last_semi=True, will raise an error if the text ends
         # with a semicolon (inserted by the user). This is the case for 
         # all symbols except the text boxes. A semi colon there would
-        # prevent a COMMENT, and would only be deteted when parsing the
-        # full model - without precise identication of the location.
+        # prevent a COMMENT, and would only be detected when parsing the
+        # full model - without precise identification of the location.
         # By doing it here we can spot the issue immediately
         try:
             _, syntax_errors, _, _, _ = self.parser.parseSingleElement(
@@ -353,7 +354,7 @@ class Symbol(QObject, QGraphicsPathItem):
     def paint(self, painter, _, ___):
         ''' Apply anti-aliasing or not (symbol attribute) '''
         painter.setRenderHint(QPainter.Antialiasing, self._antialiasing)
-        super(Symbol, self).paint(painter, _, ___)
+        super().paint(painter, _, ___)
 
     def update_position(self):
         ''' VIRTUAL - implemented in subclasses '''
@@ -486,7 +487,7 @@ class Symbol(QObject, QGraphicsPathItem):
         if not hasattr(self, '_no_hyperlink'):
             my_menu.addAction(hl_action)
         my_menu.addAction(png_action)
-        action = my_menu.exec_(event.screenPos())
+        action = my_menu.exec(event.screenPos())
         if action:
             if action.text() == png_action:
                 # Save a picture of the selected symbol and all its children
@@ -514,7 +515,7 @@ class Symbol(QObject, QGraphicsPathItem):
         ''' resize item, e.g. when editing text - move children accordingly '''
         # Call stack:
         # scene_refresh() => Text_Interaction.try_resize() => resize_item()
-        LOG.debug("resize item")
+        #LOG.debug("resize item")
         #LOG.debug(traceback.print_stack())
         if not self.resizeable:
             return
@@ -550,7 +551,8 @@ class Symbol(QObject, QGraphicsPathItem):
             pass
         try:
             self.branch_entrypoint.parent.update_connections()
-        except AttributeError:
+        except AttributeError as err:
+            # All symbols that have no branch entrypoint
             pass
 
     def set_shape(self, width, height):
@@ -576,11 +578,13 @@ class Symbol(QObject, QGraphicsPathItem):
         self.coord = self.position
         rect = self.boundingRect()
         self.height = rect.height()
-        if self.grabber.resize_mode != '':
-            self.mode = 'Resize'
-            self.old_rect = self.boundingRect()
-        else:
-            self.mode = 'Move'
+#       if self.grabber.resize_mode != '':
+#           self.mode = 'Resize'
+#           self.old_rect = self.boundingRect()
+#       else:
+        # Manual resizing is a disabled function, as the correct
+        # size for items is computed automatically on the fly
+        self.mode = 'Move'
 
     def double_click(self):
         ''' Handle double click on symbol - redefined at symbol level '''
@@ -633,8 +637,10 @@ class Symbol(QObject, QGraphicsPathItem):
                 self.cam(self.coord, self.position)
                 # Emit signal to indicate that the symbol moved
                 # typically caught by connectors
-                self.moved.emit(self.coord.x() - self.pos_x,
-                                self.coord.y() - self.pos_y)
+                # Moved to sdlSymbols.Process class, this event is actually
+                # sent while moving, so that connection is updated on the fly
+                #self.moved.emit(self.coord.x() - self.pos_x,
+                #                self.coord.y() - self.pos_y)
         self.mode = ''
 
     def updateConnectionPoints(self):
@@ -654,20 +660,27 @@ class Symbol(QObject, QGraphicsPathItem):
         while top_level.hasParent:
             # The "or top_level.parent" below is due to a Pyside/Qt bug
             # of the parentItem() function. It can happen that even when
-            # the parent has explicitely been set with "setParentItem",
+            # the parent has explicitly been set with "setParentItem",
             # a subsequent call to parentItem returns None. Seems to happen
             # if the parent has not been added yet to the scene.
             top_level = top_level.parentItem() or top_level.parent
         return top_level
 
+    def cam_group(self):
+        ''' Set the graphical boundaries of the item to apply the CAM on
+            This can be redefined in subclasses, for example to exclude
+            connections '''
+        return (self.sceneBoundingRect() |
+                self.mapRectToScene(self.childrenBoundingRect()))
+
     # pylint: disable=R0914
     def cam(self, old_pos, new_pos, ignore=None):
         ''' Collision Avoidance Manoeuvre for top level symbols '''
         # Since the cam function is recursive it may be time consuming
-        # Call the Qt event prcessing to avoid blocking the application
+        # Call the Qt event processing to avoid blocking the application
         # Removed (had bad visual side effects)
         # QApplication.processEvents()
-        #print 'CAM', str(self)[slice(0, 20)]
+        #print ('CAM', str(self)[slice(0, 20)])
         ignore = ignore or []
         if not self.scene():
             # Make sure the item is in a scene. For instance, when loading
@@ -677,7 +690,7 @@ class Symbol(QObject, QGraphicsPathItem):
 
         top_level = self.top_level()
         if top_level != self:
-            # Exectute CAM on top level of this item
+            # Execute CAM on top level of this item
             top_level.cam(top_level.position, top_level.position)
             return
 
@@ -685,8 +698,7 @@ class Symbol(QObject, QGraphicsPathItem):
         delta = new_pos - old_pos
 
         # Rectangle of current group of item in scene coordinates
-        rect = (self.sceneBoundingRect() |
-                self.mapRectToScene(self.childrenBoundingRect()))
+        rect = self.cam_group()
 
         # Move the rectangle to the new position, and move the current item
         animation = False
@@ -771,14 +783,17 @@ class Comment(Symbol):
     redbold = ()
     textbox_alignment = Qt.AlignLeft | Qt.AlignVCenter
 
+    # Minimum size for symbol
+    min_size = (70, 35)
+
     def __init__(self, parent=None, ast=None):
         ast = ast or ogAST.Comment()
         self.ast = ast
-        super(Comment, self).__init__(parent)
+        super().__init__(parent)
         self.connection = None
         self.set_shape(ast.width, ast.height)
         self.text = EditableText(parent=self,
-                                 text=ast.inputString,
+                                 text=ast.inputString.replace("\\'", "'"),  # remove escaping
                                  hyperlink=ast.hyperlink)
         if parent:
             local_pos = parent.mapFromScene(ast.pos_x or 0, ast.pos_y or 0)
@@ -799,7 +814,7 @@ class Comment(Symbol):
         if not parent:
             return
         parent.comment = self
-        super(Comment, self).insert_symbol(parent, x, y)
+        super().insert_symbol(parent, x, y)
         self.pos_x = x if x is not None else parent.boundingRect().width() + 20
         self.pos_y = y if y is not None else (parent.boundingRect().height() -
                                               self.boundingRect().height()) / 2
@@ -809,7 +824,13 @@ class Comment(Symbol):
         except AttributeError:
             # if called before text is initialized
             pass
-        #parent.cam(parent.position, parent.position)
+        parent.cam(parent.position, parent.position)
+
+    def cam(self, old_pos, new_pos, ignore=None):
+        ''' Comment is a top-level symbol - CAM would not apply properly
+        to siblings in a decision branch. We must apply the CAM to the
+        parent symbol to which the comment is attached. '''
+        self.parent.cam(self.parent.position, self.parent.position)
 
     def connect_to_parent(self):
         ''' Redefinition of the function to use a comment connector '''
@@ -821,7 +842,7 @@ class Comment(Symbol):
             reset comment field in parent
         '''
         self.parentItem().comment = None
-        super(Comment, self).delete_symbol()
+        super().delete_symbol()
 
     def resize_item(self, rect):
         '''
@@ -839,7 +860,7 @@ class Comment(Symbol):
         path = QPainterPath()
         path.addRect(0, 0, width, height)
         self.setPath(path)
-        super(Comment, self).set_shape(width, height)
+        super().set_shape(width, height)
 
     def paint(self, painter, _, ___):
         ''' Draw the comment symbol '''
@@ -848,6 +869,8 @@ class Comment(Symbol):
         pen.setStyle(Qt.DashLine)
         pen.setColor(Qt.darkGray)
         painter.setPen(pen)
+        if self.text.hasFocus():
+            painter.fillRect(rect, Qt.white)
         x, y, w, h = rect.x(), rect.y(), rect.width(), rect.height()
         if self.on_the_right:
             painter.drawLines([QPoint(w, y), QPoint(x, y),
@@ -859,8 +882,8 @@ class Comment(Symbol):
                                QPoint(w, h), QPoint(x, h)])
 
     def mouse_move(self, event):
-        ''' Handle item move '''
-        super(Comment, self).mouse_move(event)
+        ''' Comment symbol: Handle item move '''
+        super().mouse_move(event)
         if self.mode == 'Move':
             self.pos_y += event.pos().y() - event.lastPos().y()
             self.pos_x += event.pos().x() - event.lastPos().x()
@@ -878,7 +901,7 @@ class Comment(Symbol):
         if not move_accepted:
             self.position = self.coord
             self.update_connections()
-        return super(Comment, self).mouse_release(event)
+        return super().mouse_release(event)
 
     def updateConnectionPoints(self):
         '''
@@ -901,7 +924,7 @@ class Cornergrabber(QGraphicsPolygonItem):
 
     def __init__(self, parent):
         ''' Create the grabber '''
-        super(Cornergrabber, self).__init__(parent)
+        super().__init__(parent)
         self.setParentItem(parent)
         self.parent = parent
         self.setFlags(QGraphicsItem.ItemIsSelectable |
@@ -929,7 +952,7 @@ class Cornergrabber(QGraphicsPolygonItem):
     def mousePressEvent(self, event):
         ''' Handle Qt event '''
         self.parent.mouse_click(event)
-        super(Cornergrabber, self).mousePressEvent(event)
+        super().mousePressEvent(event)
 
     def mouseMoveEvent(self, event):
         ''' Handle Qt event '''
@@ -948,37 +971,39 @@ class Cornergrabber(QGraphicsPolygonItem):
             cursor = self.parent.cursor()
         elif self.parent.in_start_zone(event.pos().toPoint()):
             cursor = Qt.CrossCursor
-        elif not self.parent.resizeable:
-            cursor = self.parent.default_cursor
-            self.resize_mode = ''
-        # Left side
-        elif 0.0 <= pos.x() <= 10.0:
-            # Top-left corner: disabled
-            #if pos.y() <= 10.0:
-            #    cursor = Qt.SizeFDiagCursor
-            if pos.y() > self.boundingRect().height() - 10.0:
-                cursor = Qt.SizeBDiagCursor
-                self.resize_mode = 'bottom_left'
-            else:
-                cursor = Qt.SizeHorCursor
-                self.resize_mode = 'left'
-        # Middle of item
-        elif 5.0 < pos.x() < self.boundingRect().width() - 10.0 and (
-                     pos.y() > self.boundingRect().height() - 10.0):
-            cursor = Qt.SizeVerCursor
-            self.resize_mode = 'bottom'
-        # Right side
-        elif(self.boundingRect().width() - 5.0 <= pos.x()
-                                <= self.boundingRect().width()):
-            # Top-right corner: disabled
-            #if pos.y() <= 10.0:
-            #    cursor = Qt.SizeBDiagCursor
-            if pos.y() > self.boundingRect().height() - 10.0:
-                cursor = Qt.SizeFDiagCursor
-                self.resize_mode = 'bottom_right'
-            else:
-                cursor = Qt.SizeHorCursor
-                self.resize_mode = 'right'
+
+        # Manual resizing is disables so corresponding cursors are not needed
+#       elif not self.parent.resizeable:
+#           cursor = self.parent.default_cursor
+#           self.resize_mode = ''
+#       # Left side
+#       elif 0.0 <= pos.x() <= 10.0:
+#           # Top-left corner: disabled
+#           #if pos.y() <= 10.0:
+#           #    cursor = Qt.SizeFDiagCursor
+#           if pos.y() > self.boundingRect().height() - 10.0:
+#               cursor = Qt.SizeBDiagCursor
+#               self.resize_mode = 'bottom_left'
+#           else:
+#               cursor = Qt.SizeHorCursor
+#               self.resize_mode = 'left'
+#       # Middle of item
+#       elif 5.0 < pos.x() < self.boundingRect().width() - 10.0 and (
+#                    pos.y() > self.boundingRect().height() - 10.0):
+#           cursor = Qt.SizeVerCursor
+#           self.resize_mode = 'bottom'
+#       # Right side
+#       elif(self.boundingRect().width() - 5.0 <= pos.x()
+#                               <= self.boundingRect().width()):
+#           # Top-right corner: disabled
+#           #if pos.y() <= 10.0:
+#           #    cursor = Qt.SizeBDiagCursor
+#           if pos.y() > self.boundingRect().height() - 10.0:
+#               cursor = Qt.SizeFDiagCursor
+#               self.resize_mode = 'bottom_right'
+#           else:
+#               cursor = Qt.SizeHorCursor
+#               self.resize_mode = 'right'
         else:
             cursor = self.parent.default_cursor
             self.resize_mode = ''
@@ -993,7 +1018,7 @@ class HorizontalSymbol(Symbol):
     '''
     def __init__(self, parent=None, text='...',
             x=None, y=None, hyperlink=None):
-        super(HorizontalSymbol, self).__init__(parent)
+        super().__init__(parent)
         self.minDistanceToSymbolAbove = 20
         self.connection = None
         if self.has_text_area:
@@ -1015,7 +1040,7 @@ class HorizontalSymbol(Symbol):
     def set_valid_pos(self, pos):
         ''' Redefined function - make sure symbol is below its parent '''
         if not self.hasParent:
-            super(HorizontalSymbol, self).set_valid_pos(pos)
+            super().set_valid_pos(pos)
         else:
             new_y = max(pos.y(), self.parent.boundingRect().height() +
                         self.minDistanceToSymbolAbove)
@@ -1026,7 +1051,7 @@ class HorizontalSymbol(Symbol):
         if not parent:
             self.position = QPointF(pos_x or 0, pos_y or 0)
             return
-        super(HorizontalSymbol, self).insert_symbol(parent, pos_x, pos_y)
+        super().insert_symbol(parent, pos_x, pos_y)
         if pos_x is None or pos_y is None:
             # Usually for first insertion when item is created:
             # compute position and (if relevant) move siblings
@@ -1054,7 +1079,7 @@ class HorizontalSymbol(Symbol):
             if has_siblings:
                 pos_x = most_left + group_width + 20
             else:
-                # Verical alignment (x-axis):
+                # Vertical alignment (x-axis):
                 pos_x = (parent.boundingRect().width() -
                         self.boundingRect().width()) / 2
             pos_y = (parent.boundingRect().height() +
@@ -1069,7 +1094,7 @@ class HorizontalSymbol(Symbol):
            Redefined from Symbol class
            Horizontal symbols may have siblings - check their shape.
         '''
-        super(HorizontalSymbol, self).update_connections()
+        super().update_connections()
         try:
             for sibling in self.siblings():
                 for cnx in sibling.last_branch_item.connections():
@@ -1097,14 +1122,14 @@ class HorizontalSymbol(Symbol):
         return None
 
     def mouse_move(self, event):
-        ''' Will prevent move from being above the parent '''
+        ''' Horizontal symbols: prevent move from being above the parent '''
         if self.mode == 'Move':
             event_pos = event.pos()
             new_y = self.pos_y + (event_pos.y() - event.lastPos().y())
             new_x = self.pos_x + (event_pos.x() - event.lastPos().x())
             self.position = QPointF(new_x, new_y)
             self.update_connections()
-        super(HorizontalSymbol, self).mouse_move(event)
+        super().mouse_move(event)
 
     def cam(self, old_pos, new_pos, ignore=None):
         '''
@@ -1160,7 +1185,7 @@ class HorizontalSymbol(Symbol):
                         # If there is no scene or no undo stack
                         pass
                     sibling.cam(sibling.position, sibling.position)
-        super(HorizontalSymbol, self).cam(old_pos, new_pos, ignore)
+        super().cam(old_pos, new_pos, ignore)
         # Recursively call the cam of the parent
         try:
             self.parentItem().cam(self.parentItem().position,
@@ -1177,7 +1202,7 @@ class VerticalSymbol(Symbol):
     '''
     def __init__(self, parent=None, text='...',
             x=None, y=None, hyperlink=None):
-        super(VerticalSymbol, self).__init__(parent)
+        super().__init__(parent)
         self.connection = None
         if self.has_text_area:
             self.text = EditableText(parent=self,
@@ -1204,7 +1229,7 @@ class VerticalSymbol(Symbol):
     def set_valid_pos(self, pos):
         ''' Redefined function - make sure symbol is well aligned '''
         if not self.hasParent:
-            super(VerticalSymbol, self).set_valid_pos(pos)
+            super().set_valid_pos(pos)
         else:
             # 'or self.parent' because of pyside/qt bug
             parent = self.parentItem() or self.parent
@@ -1228,7 +1253,7 @@ class VerticalSymbol(Symbol):
             if x is not None and y is not None:
                 self.position = QPointF(x, y)
             return
-        super(VerticalSymbol, self).insert_symbol(parent, x, y)
+        super().insert_symbol(parent, x, y)
         # in a branch (e.g. DECISION) all items must know the first element
         # (used for computing the branch size and the connection point)
         self.branch_entrypoint = parent.branch_entrypoint
@@ -1296,7 +1321,7 @@ class VerticalSymbol(Symbol):
 
     def mouse_move(self, event):
         ''' Click and move: forbid symbol to move on the x axis '''
-        super(VerticalSymbol, self).mouse_move(event)
+        super().mouse_move(event)
         if self.mode == 'Move':
             new_y = self.pos_y + event.pos().y() - event.lastPos().y()
             new_x = self.pos_x + event.pos().x() - event.lastPos().x()
@@ -1315,4 +1340,4 @@ class VerticalSymbol(Symbol):
                 branch_entry = branch_entry.parentItem() or branch_entry.parent
             branch_entry.cam(branch_entry.position, branch_entry.position)
         else:
-            super(VerticalSymbol, self).cam(old_pos, new_pos, ignore)
+            super().cam(old_pos, new_pos, ignore)
